@@ -2,14 +2,7 @@ import pytest
 from aiconfig.Config import AIConfigRuntime
 from aiconfig.util.config_utils import extract_override_settings
 
-from aiconfig.schema import (
-    AIConfig,
-    ConfigMetadata,
-    ExecuteResult,
-    ModelMetadata,
-    Prompt,
-    PromptMetadata,
-)
+from aiconfig.schema import AIConfig, ConfigMetadata, ExecuteResult, ModelMetadata, Prompt, PromptMetadata
 
 
 @pytest.fixture
@@ -68,9 +61,7 @@ def test_delete_nonexistent_model(ai_config: AIConfig):
     non_existent_model = "non_existent_model"
 
     # Ensure trying to delete a non-existent model raises an exception
-    with pytest.raises(
-        Exception, match=f"Model '{non_existent_model}' does not exist."
-    ):
+    with pytest.raises(Exception, match=f"Model '{non_existent_model}' does not exist."):
         ai_config.delete_model(non_existent_model)
 
 
@@ -201,6 +192,7 @@ def test_update_nonexistent_prompt(ai_config_runtime: AIConfigRuntime):
     Test updating a nonexistent prompt.
     """
     config = ai_config_runtime
+    nonexistent_prompt_name = "nonexistent_prompt"
 
     prompt_data = Prompt(
         name="prompt1",
@@ -209,8 +201,8 @@ def test_update_nonexistent_prompt(ai_config_runtime: AIConfigRuntime):
     )
 
     # Ensure updating a nonexistent prompt raises an exception
-    with pytest.raises(IndexError, match=r"Prompt not found in config"):
-        config.update_prompt("nonexistent_prompt", prompt_data)
+    with pytest.raises(IndexError, match=f"Prompt '{nonexistent_prompt_name}' not found in config"):
+        config.update_prompt(nonexistent_prompt_name, prompt_data)
 
 
 def test_delete_nonexistent_prompt(ai_config_runtime: AIConfigRuntime):
@@ -218,10 +210,11 @@ def test_delete_nonexistent_prompt(ai_config_runtime: AIConfigRuntime):
     Test deleting a nonexistent prompt.
     """
     config = ai_config_runtime
+    prompt_name = "nonexistent_prompt"
 
     # Ensure deleting a nonexistent prompt raises an exception
-    with pytest.raises(IndexError, match=r"Prompt not found in config"):
-        config.delete_prompt("nonexistent_prompt")
+    with pytest.raises(IndexError, match=f"Prompt '{prompt_name}' not found in config"):
+        config.delete_prompt(prompt_name)
 
 
 def test_get_metadata_with_nonexistent_prompt(ai_config_runtime: AIConfigRuntime):
@@ -236,28 +229,6 @@ def test_get_metadata_with_nonexistent_prompt(ai_config_runtime: AIConfigRuntime
         config.get_metadata(prompt_name)
 
 
-def test_delete_nonexistent_parameter(ai_config_runtime: AIConfigRuntime):
-    """
-    Test deleting a nonexistent parameter.
-    """
-    config = ai_config_runtime
-    parameter_name_to_delete = "param1"
-    config.add_prompt(
-        "prompt1",
-        Prompt(
-            name="prompt_name",
-            input="This is a prompt",
-            metadata=PromptMetadata(model="fakemodel"),
-        ),
-    )
-
-    # Ensure deleting a nonexistent parameter raises a KeyError
-    with pytest.raises(
-        KeyError, match=f"Parameter '{parameter_name_to_delete}' does not exist."
-    ):
-        config.delete_parameter(parameter_name_to_delete)
-
-
 @pytest.fixture
 def ai_config():
     config = AIConfig(
@@ -269,74 +240,6 @@ def ai_config():
     return config
 
 
-def test_set_global_parameter(ai_config: AIConfig):
-    """
-    Test setting a global parameter.
-    """
-    parameter_name = "global_param"
-    parameter_value = "global_value"
-
-    ai_config.set_parameter(parameter_name, parameter_value, prompt_name=None)
-
-    # Ensure the global parameter is set correctly
-    assert ai_config.metadata.parameters[parameter_name] == parameter_value
-
-
-def test_set_parameter_for_prompt(ai_config: AIConfig):
-    """
-    Test setting a parameter for a specific prompt.
-    """
-    prompt_name = "prompt1"
-    parameter_name = "prompt_param"
-    parameter_value = "prompt_value"
-
-    # Create a sample prompt for testing
-    prompt_data = Prompt(
-        name=prompt_name,
-        input="This is a prompt",
-        metadata=PromptMetadata(model="fakemodel"),
-    )
-    ai_config.add_prompt(prompt_name, prompt_data)
-
-    ai_config.set_parameter(parameter_name, parameter_value, prompt_name=prompt_name)
-
-    # Ensure the parameter is set for the specific prompt
-    assert (
-        ai_config.prompt_index[prompt_name].metadata.parameters[parameter_name]
-        == parameter_value
-    )
-    assert ai_config.prompts[0].metadata.parameters[parameter_name] == parameter_value
-
-
-def test_update_existing_parameter(ai_config: AIConfig):
-    """
-    Test updating an existing parameter.
-    """
-    parameter_name = "existing_param"
-    initial_value = "initial_value"
-    updated_value = "updated_value"
-
-    ai_config.set_parameter(parameter_name, initial_value, prompt_name=None)
-    ai_config.update_parameter(parameter_name, updated_value, prompt_name=None)
-
-    # Ensure the existing parameter is updated correctly
-    assert ai_config.metadata.parameters[parameter_name] == updated_value
-
-
-def test_delete_existing_parameter(ai_config: AIConfig):
-    """
-    Test deleting an existing parameter.
-    """
-    parameter_name_to_delete = "param_to_delete"
-    parameter_value = "param_value"
-
-    ai_config.set_parameter(parameter_name_to_delete, parameter_value, prompt_name=None)
-    ai_config.delete_parameter(parameter_name_to_delete, prompt_name=None)
-
-    # Ensure the existing parameter is deleted correctly
-    assert parameter_name_to_delete not in ai_config.metadata.parameters
-
-
 def test_load_saved_config(tmp_path):
     """
     Test loading a saved AIConfig from a JSON file.
@@ -366,9 +269,8 @@ def test_load_saved_config(tmp_path):
     assert loaded_config.name == "My AIConfig"
     assert loaded_config.metadata.parameters == {"config_param": "config_value"}
     assert "prompt1" in loaded_config.prompt_index
-    assert loaded_config.prompt_index["prompt1"].metadata.parameters == {
-        "prompt_param": "prompt_value"
-    }
+    assert loaded_config.prompt_index["prompt1"].metadata is not None
+    assert loaded_config.prompt_index["prompt1"].metadata.parameters == {"prompt_param": "prompt_value"}
 
 
 def test_set_config_name(ai_config_runtime: AIConfigRuntime):
@@ -412,55 +314,109 @@ def test_get_prompt_after_deleting_previous(ai_config_runtime: AIConfigRuntime):
 
 
 def test_get_prompt_nonexistent(ai_config_runtime: AIConfigRuntime):
-    with pytest.raises(
-        IndexError, match=r"Prompt 'GreetingPrompt' not found in config"
-    ):
+    with pytest.raises(IndexError, match=r"Prompt 'GreetingPrompt' not found in config"):
         ai_config_runtime.get_prompt("GreetingPrompt")
 
 
-def test_update_model_ai_config(ai_config_runtime: AIConfigRuntime):
-    model_metadata = {"name": "testmodel", "settings": {"topP": 0.9}}
-    ai_config_runtime.update_model(model_metadata)
-    assert (
-        ai_config_runtime.metadata.models["testmodel"]
-        == ModelMetadata(**{"name": "testmodel", "settings": {"topP": 0.9}}).settings
-    )
+def test_update_model_for_ai_config(ai_config_runtime: AIConfigRuntime):
+    """Test updating model without a specific prompt."""
+    # New model name, new settings --> update
+    model_name = "testmodel"
+    settings = {"topP": 0.9}
+    ai_config_runtime.update_model(model_name, settings)
+    pytest.warns(match=f"No prompt name was given to update the model name to '{model_name}'.")
+    assert ai_config_runtime.metadata.models is not None
+    assert ai_config_runtime.metadata.models[model_name] == settings
+
+    # Existing model name, no settings --> no-op
+    ai_config_runtime.update_model(model_name)
+    pytest.warns(match=f"No prompt name was given to update the model name to '{model_name}'.")
+    assert ai_config_runtime.metadata.models is not None
+    assert ai_config_runtime.metadata.models[model_name] == settings
+
+    # Existing model name, new settings --> update
+    new_settings = {"topP": 0.75}
+    ai_config_runtime.update_model(model_name, new_settings)
+    pytest.warns(match=f"No prompt name was given to update the model name to '{model_name}'.")
+    assert ai_config_runtime.metadata.models is not None
+    assert ai_config_runtime.metadata.models[model_name] == new_settings
+
+    # New model name, no settings --> update
+    new_model_name = "testmodel_without_settings"
+    ai_config_runtime.update_model(new_model_name)
+    pytest.warns(match=f"No prompt name was given to update the model name to '{new_model_name}'.")
+    assert ai_config_runtime.metadata.models is not None
+    assert ai_config_runtime.metadata.models[new_model_name] == {}
 
 
-def test_update_model_specific_prompt(ai_config_runtime: AIConfigRuntime):
-    """Test updating model metadata for a specific prompt."""
-    model_metadata = {"name": "testmodel", "settings": {"topP": 0.9}}
-    prompt1 = Prompt(
-        name="GreetingPrompt",
-        input="Hello, how are you?",
-        metadata=PromptMetadata(model="fakemodel"),
-    )
+def test_update_model_for_prompt(ai_config_runtime: AIConfigRuntime):
+    """Test updating model for a specific prompt."""
+    # New model name, new settings, no prompt metadata --> update
+    prompt1 = Prompt(name="GreetingPrompt", input="Hello, how are you?")
     ai_config_runtime.add_prompt(prompt1.name, prompt1)
-    ai_config_runtime.update_model(model_metadata, "GreetingPrompt")
-    assert ai_config_runtime.get_prompt(
-        "GreetingPrompt"
-    ).metadata.model == ModelMetadata(**model_metadata)
+    model_name = "testmodel"
+    settings = {"topP": 0.9}
+    ai_config_runtime.update_model(model_name, settings, prompt1.name)
+    prompt = ai_config_runtime.get_prompt(prompt1.name)
+    assert prompt.metadata is not None
+    assert prompt.metadata.model == ModelMetadata(name=model_name, settings=settings)
+
+    # New model name, no settings --> update name only
+    new_model_name = "testmodel_new_name"
+    ai_config_runtime.update_model(new_model_name, None, prompt1.name)
+    prompt = ai_config_runtime.get_prompt(prompt1.name)
+    assert prompt.metadata is not None
+    assert prompt.metadata.model == ModelMetadata(name=new_model_name, settings=settings)
+
+    # Same model name, new settings --> update settings only
+    settings = {"topP": 0.9}
+    ai_config_runtime.update_model(new_model_name, settings, prompt1.name)
+    prompt = ai_config_runtime.get_prompt(prompt1.name)
+    assert prompt.metadata is not None
+    assert prompt.metadata.model == ModelMetadata(name=new_model_name, settings=settings)
+
+    # New name, no settings, prompt with model name as string --> update
+    prompt2 = Prompt(
+        name="GreetingPromptModelAsStr",
+        input="Hello, how are you?",
+        metadata=PromptMetadata(model="some_random_model"),
+    )
+    ai_config_runtime.add_prompt(prompt2.name, prompt2)
+    new_name_again = "testmodel_new_name_model_as_str"
+    ai_config_runtime.update_model(new_name_again, None, prompt2.name)
+    prompt = ai_config_runtime.get_prompt(prompt2.name)
+    assert prompt.metadata is not None
+    assert prompt.metadata.model == ModelMetadata(name=new_name_again, settings={})
+
+    # New name, no settings, prompt with metadata but no model --> update
+    tags = ["my_fancy_tags"]
+    prompt3 = Prompt(
+        name="GreetingsNumber3",
+        input="Hello, how are you?",
+        metadata=PromptMetadata(tags=tags),
+    )
+    ai_config_runtime.add_prompt(prompt3.name, prompt3)
+    new_name_3 = "new_name_number_3"
+    ai_config_runtime.update_model(new_name_3, None, prompt3.name)
+    prompt = ai_config_runtime.get_prompt(prompt3.name)
+    assert prompt.metadata is not None
+    assert prompt.metadata.model == ModelMetadata(name=new_name_3, settings={})
+    assert prompt.metadata.tags == tags
 
 
-def test_update_model_empty_metadata(ai_config_runtime: AIConfigRuntime):
-    """Test updating with an empty model_metadata dictionary."""
-    model_metadata = {}
+def test_update_model_with_invalid_arguments(ai_config_runtime: AIConfigRuntime):
+    """Test trying to update model with invalid arguments."""
+    with pytest.raises(
+        ValueError,
+        match=r"Cannot update model. Either model name or model settings must be specified.",
+    ):
+        ai_config_runtime.update_model(model_name=None, settings=None)
 
     with pytest.raises(
-        KeyError,
-        match=r"Cannot update model. Model metadata must contain a 'name' element. Optionally, it may contain a 'settings' element.",
+        ValueError,
+        match=r"Cannot update model. There are two things you are trying:",
     ):
-        ai_config_runtime.update_model(model_metadata)
-
-
-def test_set_metadata_ai_config(ai_config_runtime: AIConfigRuntime):
-    """Test setting metadata at the AIConfig level."""
-    model_metadata = {"name": "testmodel", "settings": {"topP": 0.9}}
-    ai_config_runtime.update_model(model_metadata)
-    assert (
-        ai_config_runtime.get_metadata().models["testmodel"]
-        == ModelMetadata(**model_metadata).settings
-    )
+        ai_config_runtime.update_model(model_name=None, settings={"top": 0.9}, prompt_name=None)
 
 
 def test_set_and_delete_metadata_ai_config(ai_config_runtime: AIConfigRuntime):
@@ -476,39 +432,34 @@ def test_set_and_delete_metadata_ai_config(ai_config_runtime: AIConfigRuntime):
 
 def test_set_and_delete_metadata_ai_config_prompt(ai_config_runtime: AIConfigRuntime):
     """Test deleting a non-existent metadata key at the AIConfig level."""
-    prompt1 = Prompt(
+    prompt = Prompt(
         name="GreetingPrompt",
         input="Hello, how are you?",
         metadata=PromptMetadata(model="fakemodel"),
     )
-    ai_config_runtime.add_prompt(prompt1.name, prompt1)
+    ai_config_runtime.add_prompt(prompt.name, prompt)
     ai_config_runtime.set_metadata("testkey", "testvalue", "GreetingPrompt")
 
-    assert (
-        ai_config_runtime.get_prompt("GreetingPrompt").metadata.testkey == "testvalue"
-    )
+    assert ai_config_runtime.get_prompt("GreetingPrompt").metadata.testkey == "testvalue"
 
     ai_config_runtime.delete_metadata("testkey", "GreetingPrompt")
 
-    assert (
-        hasattr(ai_config_runtime.get_prompt("GreetingPrompt").metadata, "testkey")
-        is False
-    )
+    assert hasattr(ai_config_runtime.get_prompt("GreetingPrompt").metadata, "testkey") is False
 
 
 def test_add_output_existing_prompt_no_overwrite(ai_config_runtime: AIConfigRuntime):
     """Test adding an output to an existing prompt without overwriting."""
-    prompt1 = Prompt(
+    prompt = Prompt(
         name="GreetingPrompt",
         input="Hello, how are you?",
         metadata=PromptMetadata(model="fakemodel"),
     )
-    ai_config_runtime.add_prompt(prompt1.name, prompt1)
+    ai_config_runtime.add_prompt(prompt.name, prompt)
     test_result = ExecuteResult(
         output_type="execute_result",
-        execution_count=0.0,
-        data={"role": "assistant", "content": "test output"},
-        metadata={"finish_reason": "stop"},
+        execution_count=0,
+        data="test output",
+        metadata={"raw_response": {"role": "assistant", "content": "test output"}},
     )
     ai_config_runtime.add_output("GreetingPrompt", test_result)
 
@@ -516,41 +467,178 @@ def test_add_output_existing_prompt_no_overwrite(ai_config_runtime: AIConfigRunt
 
     test_result2 = ExecuteResult(
         output_type="execute_result",
-        execution_count=0.0,
-        data={"role": "assistant", "content": "test output"},
-        metadata={"finish_reason": "stop"},
+        execution_count=0,
+        data="test output",
+        metadata={"raw_response": {"role": "assistant", "content": "test output for second time"}},
     )
+
     ai_config_runtime.add_output("GreetingPrompt", test_result2)
     assert ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
 
     ai_config_runtime.delete_output("GreetingPrompt")
-
     assert ai_config_runtime.get_latest_output("GreetingPrompt") == None
 
 
+def test_add_outputs_existing_prompt_no_overwrite(ai_config_runtime: AIConfigRuntime):
+    """Test adding outputs to an existing prompt without overwriting."""
+    original_result = ExecuteResult(
+        output_type="execute_result",
+        execution_count=0,
+        data="original result",
+        metadata={"raw_response": {"role": "assistant", "content": "original result"}},
+    )
+    prompt = Prompt(
+        name="GreetingPrompt",
+        input="Hello, how are you?",
+        metadata=PromptMetadata(model="fakemodel"),
+        outputs=[original_result],
+    )
+    ai_config_runtime.add_prompt(prompt.name, prompt)
+
+    assert ai_config_runtime.get_latest_output("GreetingPrompt") == original_result
+
+    test_result1 = ExecuteResult(
+        output_type="execute_result",
+        execution_count=0,
+        data="test output 1",
+        metadata={"raw_response": {"role": "assistant", "content": "test output 1"}},
+    )
+    test_result2 = ExecuteResult(
+        output_type="execute_result",
+        execution_count=0,
+        data="test output 2",
+        metadata={"raw_response": {"role": "assistant", "content": "test output 2"}},
+    )
+    ai_config_runtime.add_outputs("GreetingPrompt", [test_result1, test_result2])
+
+    assert ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
+    assert prompt.outputs == [original_result, test_result1, test_result2]
+
+
+def test_add_outputs_existing_prompt_with_overwrite(ai_config_runtime: AIConfigRuntime):
+    """Test adding outputs to an existing prompt with overwriting."""
+    original_result = ExecuteResult(
+        output_type="execute_result",
+        execution_count=0,
+        data="original result",
+        metadata={"raw_response": {"role": "assistant", "content": "original result"}},
+    )
+    prompt = Prompt(
+        name="GreetingPrompt",
+        input="Hello, how are you?",
+        metadata=PromptMetadata(model="fakemodel"),
+        outputs=[original_result],
+    )
+    ai_config_runtime.add_prompt(prompt.name, prompt)
+
+    assert ai_config_runtime.get_latest_output("GreetingPrompt") == original_result
+
+    test_result1 = ExecuteResult(
+        output_type="execute_result",
+        execution_count=0,
+        data="test output 1",
+        metadata={"raw_response": {"role": "assistant", "content": "test output 1"}},
+    )
+    test_result2 = ExecuteResult(
+        output_type="execute_result",
+        execution_count=0,
+        data="test output 2",
+        metadata={"raw_response": {"role": "assistant", "content": "test output 2"}},
+    )
+    ai_config_runtime.add_outputs("GreetingPrompt", [test_result1, test_result2], True)
+
+    assert ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
+    assert prompt.outputs == [test_result1, test_result2]
+
+
+def test_add_undefined_outputs_to_prompt(ai_config_runtime: AIConfigRuntime):
+    """Test for adding undefined outputs to an existing prompt with/without overwriting. Should result in an error."""
+    prompt = Prompt(
+        name="GreetingPrompt",
+        input="Hello, how are you?",
+        metadata=PromptMetadata(model="fakemodel"),
+    )
+    ai_config_runtime.add_prompt(prompt.name, prompt)
+    assert ai_config_runtime.get_latest_output("GreetingPrompt") == None
+    # Case 1: No outputs, overwrite param not defined
+    with pytest.raises(
+        ValueError,
+        match=r"Cannot add outputs. No outputs provided for prompt 'GreetingPrompt'.",
+    ):
+        ai_config_runtime.add_outputs("GreetingPrompt", [])
+    # Case 2: No outputs, overwrite param set to True
+    with pytest.raises(
+        ValueError,
+        match=r"Cannot add outputs. No outputs provided for prompt 'GreetingPrompt'.",
+    ):
+        ai_config_runtime.add_outputs("GreetingPrompt", [], True)
+
+
+def test_add_output_existing_prompt_overwrite(ai_config_runtime: AIConfigRuntime):
+    """Test adding an output to an existing prompt with overwriting."""
+    original_output = ExecuteResult(
+        output_type="execute_result",
+        execution_count=0,
+        data="original output",
+        metadata={"raw_response": {"role": "assistant", "content": "original output"}},
+    )
+    prompt = Prompt(
+        name="GreetingPrompt",
+        input="Hello, how are you?",
+        metadata=PromptMetadata(model="fakemodel"),
+        outputs=[original_output],
+    )
+    ai_config_runtime.add_prompt(prompt.name, prompt)
+    # check that the original_output is there
+    assert ai_config_runtime.get_latest_output("GreetingPrompt") == original_output
+    expected_output = ExecuteResult(
+        output_type="execute_result",
+        execution_count=0,
+        data="original output",
+        metadata={"raw_response": {"role": "assistant", "content": "original output"}},
+    )
+    # overwrite the original_output
+    ai_config_runtime.add_output("GreetingPrompt", expected_output, True)
+    assert ai_config_runtime.get_latest_output("GreetingPrompt") == expected_output
+
+
+def test_add_undefined_output_to_prompt(ai_config_runtime: AIConfigRuntime):
+    """Test for adding an undefined output to a prompt with/without overwriting. Should result in an error."""
+    prompt = Prompt(
+        name="GreetingPrompt",
+        input="Hello, how are you?",
+        metadata=PromptMetadata(model="fakemodel"),
+    )
+    ai_config_runtime.add_prompt(prompt.name, prompt)
+    # Case 1: No output, overwrite param not defined
+    with pytest.raises(
+        ValueError,
+        match=r"Cannot add output to prompt 'GreetingPrompt'. Output is not defined.",
+    ):
+        ai_config_runtime.add_output("GreetingPrompt", None)
+    # Case 2: No output, overwrite param set to True
+    with pytest.raises(
+        ValueError,
+        match=r"Cannot add output to prompt 'GreetingPrompt'. Output is not defined.",
+    ):
+        ai_config_runtime.add_output("GreetingPrompt", None, True)
+
+
 def test_extract_override_settings(ai_config_runtime: AIConfigRuntime):
     initial_settings = {"topP": 0.9}
 
     # Test Case 1: No global setting, Expect an override
-    override = extract_override_settings(
-        ai_config_runtime, initial_settings, "testmodel"
-    )
+    override = extract_override_settings(ai_config_runtime, initial_settings, "testmodel")
     assert override == {"topP": 0.9}
 
     # Test Case 2: Global Settings differ, expect override
     ai_config_runtime.add_model("testmodel", {"topP": 0.8})
-    override = extract_override_settings(
-        ai_config_runtime, initial_settings, "testmodel"
-    )
+    override = extract_override_settings(ai_config_runtime, initial_settings, "testmodel")
     assert override == {"topP": 0.9}
 
     # Test Case 3: Global Settings match settings, expect no override
-    ai_config_runtime.update_model(
-        ModelMetadata(name="testmodel", settings={"topP": 0.9})
-    )
-    override = extract_override_settings(
-        ai_config_runtime, initial_settings, "testmodel"
-    )
+    ai_config_runtime.update_model(model_name="testmodel", settings={"topP": 0.9})
+    override = extract_override_settings(ai_config_runtime, initial_settings, "testmodel")
     assert override == {}
 
     # Test Case 4: Global settings defined and empty settings defined. Expect no override
